//
// SDLAudio.cpp
// Copyright (c) Michael J Stokes, 2007.  All rights reserved.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "Common.h"
#include "SDLAudio.h"


////////////////////////////////////////////////////////////////////////////////
// SDLAudio - construction

SDLAudio::SDLAudio()
{
	_buffer = NULL;
}

////////////////////////////////////////////////////////////////////////////////
// SDLAudio - destruction

SDLAudio::~SDLAudio()
{
	Terminate();
}

////////////////////////////////////////////////////////////////////////////////
// SDLAudio - initialize

bool SDLAudio::Initialize()
{
	if ( _active ) return true;
	
	if ( 0 == SDL_WasInit( SDL_INIT_AUDIO ) )
	{
		if ( 0 != SDL_InitSubSystem( SDL_INIT_AUDIO ) )
		{
			cerr << "Audio is unavailable" << endl;
			return false;
		}
	}
	
	memset( &_format, 0, sizeof( _format ) );
	
	_format.freq		= 44100;
	_format.format		= AUDIO_S16;
	_format.channels	= 2;
	_format.samples		= 1024;
	_format.callback	= OnMix;
	_format.userdata	= this;
	
	if ( 0 != SDL_OpenAudio( &_format, NULL ) )
	{
		cerr << "Audio is unavailable during setup" << endl;
		return false;
	}
	
	_buffer = new int[ _format.samples * 2 ];
	
	return Audio::Initialize();
}

////////////////////////////////////////////////////////////////////////////////
// SDLAudio - terminate

void SDLAudio::Terminate()
{
	if ( _active )
	{
		SDL_PauseAudio( 1 );
		SDL_CloseAudio();
		delete [] _buffer;
		Audio::Terminate();
	}
}

////////////////////////////////////////////////////////////////////////////////
// SDLAudio - lock

void SDLAudio::Lock()
{
	if ( _active ) SDL_LockAudio();
}

////////////////////////////////////////////////////////////////////////////////
// SDLAudio - unlock

void SDLAudio::Unlock()
{
	if ( _active ) SDL_UnlockAudio();
}

////////////////////////////////////////////////////////////////////////////////
// SDLAudio - pause and resume

void SDLAudio::Pause( bool pause )
{
	if ( _active )
	{
		SDL_PauseAudio( pause ? 1 : 0 );
	}
}

////////////////////////////////////////////////////////////////////////////////
// SDLAudio - mix callback

void SDLAudio::OnMix( void *self, Uint8 *stream, int length )
{
	SDLAudio* sdl = (SDLAudio*)self;
	
	UINT samples = (UINT)length >> 2;
	sdl->Process( sdl->_buffer, samples );
	
	int*	input	= sdl->_buffer;
	short*	output	= (short*)stream;
	
	for ( UINT count = samples * 2 ; count -- ; )
	{
		int sample = *input ++;
		
		if ( sample < -32760 )
			*output ++ = -32760;
		else if ( sample > 32760 )
			*output ++ = 32760;
		else
			*output ++ = (short)sample;
	}
}
