//
// VRJuggler.cpp
// Copyright Michael J Stokes, 2008.
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "Common.h"
#include "Application.h"
#include "VRJuggler.h"

#ifdef WITH_VRJUGGLER


////////////////////////////////////////////////////////////////////////////////
// VRJuggler construction

VRJuggler::VRJuggler()
{
	_app	= NULL;
	_time	= 0;
}

////////////////////////////////////////////////////////////////////////////////
// VRJuggler destruction

VRJuggler::~VRJuggler()
{
	if ( NULL != _app ) delete _app;
}

////////////////////////////////////////////////////////////////////////////////
// VRJuggler initialize

void VRJuggler::init()
{
	vrj::GlApp::init();
	_head.init( "VJWand" );
	_forward.init( "VJButton0" );
	_backward.init( "VJButton2" );
}

////////////////////////////////////////////////////////////////////////////////
// VRJuggler OpenGL context initialize

void VRJuggler::contextInit()
{
	vrj::GlApp::contextInit();
	
	if ( NULL == _app )
	{
		_app = new Application();
		if ( ! _app->Initialize() ) exit();
	}
	
	if ( ! _app->PrepareContext() ) exit();
}

////////////////////////////////////////////////////////////////////////////////
// VRJuggler pre-frame update

void VRJuggler::preFrame()
{
	vrj::GlApp::preFrame();
	
	if ( NULL != _app )
	{
		gmtl::Matrix44f gmtlHead( _head->getData( gadget::PositionUnitConversion::ConvertToMeters ) );
		matrix4x4 head( gmtlHead.getData() );
		vec3 direction( head.Direction() );
		direction.x *= -1;
		
		UINT actions = 0;
		if ( _forward->getData() ) actions |= Application::ACTION_FORWARD;
		if ( _backward->getData() ) actions |= Application::ACTION_BACKWARD;
		
#ifdef _WIN32
		DWORD now = GetTickCount();
#else
#error implement_me
#endif
		
		if ( 0 == _time ) _time = now;
		_app->Update( (scalar)( now - _time ) / 1000.0f, direction, actions );
	}
}

////////////////////////////////////////////////////////////////////////////////
// VRJuggler buffer pre-draw

void VRJuggler::bufferPreDraw()
{
	vrj::GlApp::bufferPreDraw();
	glClear( GL_COLOR_BUFFER_BIT );
}

////////////////////////////////////////////////////////////////////////////////
// VRJuggler draw

void VRJuggler::draw()
{
	glMatrixMode( GL_PROJECTION );
	glPushMatrix();
	glScalef( 1, 1, -1 );
	glMatrixMode( GL_MODELVIEW );
	glPushMatrix();
	
	glClear( GL_DEPTH_BUFFER_BIT );
	if ( NULL != _app ) _app->Render();
	
	glPopMatrix();
	glMatrixMode( GL_PROJECTION );
	glPopMatrix();
	glMatrixMode( GL_MODELVIEW );
}

////////////////////////////////////////////////////////////////////////////////
// Main

int main( int argc, char** argv )
{
	vrj::Kernel* kernel	= vrj::Kernel::instance();
	VRJuggler* app		= new VRJuggler();
	
#ifndef VRJ_USE_COCOA
	if ( argc <= 1 )
	{
		cerr << "Usage: " << argv[0] << " configfile [configfile] ..." << endl;
		return 1;
	}
#endif
	
	for ( int i = 1 ; i < argc ; i ++ )
	{
		kernel->loadConfigFile( argv[ i ] );
	}
	
	kernel->start();
	kernel->setApplication( app );
	kernel->waitForKernelStop();
	
	delete app;
	
	return 0;
}

#endif
