//
// Actor.h
// Copyright Michael J Stokes, 2009.
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#pragma once

#include "Actors.h"

class Character;
class Emitter;


class Actor
{
// Construction
public:
	Actor( const string& name, XMLElement* request );
	virtual ~Actor();
	
// Sound channel
public:
	enum
	{
		SOUND_DEFAULT	= 1,
		SOUND_VOICE		= 2,
		SOUND_FEET		= 4,
	};
	
// Inlines
public:
	inline const string& GetName() const
	{
		return _name;
	}
	
	inline int GetID() const
	{
		return _id;
	}
	
	inline void SetID( int id )
	{
		_id = id;
	}
	
	inline const vec3& GetPosition() const
	{
		return _position;
	}
	
	inline scalar GetHeading() const
	{
		return _heading;
	}
	
// Operations
public:
	XMLElement*			Send( XMLElement* request );
	void				BoneBusConnected();
	void				BoneBusRotations( int count, const Actors::BulkBoneRotation* bone );
	void				BoneBusPositions( int count, const Actors::BulkBonePosition* bone );
	void				Update();
	void				Render();
	
// Implementation
protected:
	Actors*				GetActors();
	void				SendImpl( XMLElement* request );
	void				SetCharacter( Character* character );
protected:
	bool				Play( PCSTR name, int channel = 1, bool loop = false, scalar volume = 1 );
	void				Silence( int channels = -1 );
	bool				IsPlaying( int channels = -1 );
protected:
	void				SetPosition( const vec3& position );
	void				UpdatePhysics();
protected:
	void				UpdateWalk();
	static scalar		GetHeading( const vec3& toward );
	void				UpdateGaze();
	
// Data
protected:
	string				_name;
	int					_id;
protected:
	Character*			_character;
	vector<vec3>		_positions;
	vector<quaternion>	_orientations;
	vector<matrix4x4>	_transforms;
protected:
	Emitter*			_emitter;
protected:
	vec3				_position;
	scalar				_heading;
	vec3				_velocity;
	scalar				_friction;
	scalar				_gravity;
	scalar				_height;
	scalar				_radius;
	NxController*		_controller;
protected:
	string				_posture;
	string				_gazeTarget;
	scalar				_gazeWander;
	scalar				_gazeDelay;
	string				_gazeLast;
	string				_walkTarget;
	scalar				_walkDistance;
};
