//
// Actors.h
// Copyright Michael J Stokes, 2009.
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#pragma once

#include "Buffer.h"

class Actor;


class Actors
{
// Construction
public:
	Actors();
	virtual ~Actors();
	
// Operations
public:
	void			Update();
	void			Render();
public:
	XMLElement*		Send( PCSTR name, XMLElement* request );
	void			RegisterPosition( PCSTR name, const vec3& position, scalar heading = -1 );
	void			SendSBM( const string& command );
public:
	Actor*			FindActor( PCSTR name );
	inline Actor*	FindActor( int id );
	
// Inlines
public:
	inline vec3 GetPosition( PCSTR name ) const
	{
		map<string,vec3>::const_iterator it = _pawns.find( name );
		if ( it == _pawns.end() ) return vec3();
		return it->second;
	}
	
	inline bool IsConnected() const
	{
		return ( INVALID_SOCKET != _client );
	}
	
	inline const list<Actor*>& GetActors() const
	{
		return _list;
	}
	
	inline const map<string,vec3>& GetPawns() const
	{
		return _pawns;
	}
	
// Implementation
protected:
	void			UpdateBoneBus();
	void			BoneBusConnected();
	void			HandleTCP( vector<string>& args );
	void			UpdatePositions();
	
// Data
protected:
	list<Actor*>		_list;
	map<string,vec3>	_pawns;
protected:
	SOCKET			_tcp;
	SOCKET			_udp;
	SOCKET			_client;
	Buffer			_transmit;
	Buffer			_receive;
public:
	static const PCSTR BONE_NAMES[];
	
// BulkBoneRotation
public:
	struct BulkBoneRotation
	{
		int					bone;
		float				w, x, y, z;
	};
	
// BulkBoneRotations
public:
	struct BulkBoneRotations
	{
		int					packet;
		int					character;
		int					count;
		int					unused;
		BulkBoneRotation	bones[ 1000 ];
	};
	
// BulkBonePosition
public:
	struct BulkBonePosition
	{
		int					bone;
		float				x, y, z;
	};
	
// BulkBonePositions
public:
	struct BulkBonePositions
	{
		int					packet;
		int					character;
		int					count;
		BulkBonePosition	bones[ 1000 ];
	};
};
