//
// Application.h
// Copyright Michael J Stokes, 2008.
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#pragma once

class Renderer;
class Audio;
class Physics;
class BoneBus;
class Environment;
class Features;
class Actors;
class Ego;
class Guide;


class Application
{
// Construction
public:
	Application();
	virtual ~Application();
	
// Enumeration
public:
	enum
	{
		ACTION_FORWARD		= 1,
		ACTION_BACKWARD		= 2
	};
	
// Inlines
public:
	static inline Application* GetInstance()
	{
		return _instance;
	}
	
	static inline void Destroy()
	{
		if ( NULL != _instance )
		{
			delete _instance;
			_instance = NULL;
		}
	}
	
	inline const string& GetPath() const
	{
		return _path;
	}
	
	inline Renderer* GetRenderer()
	{
		if ( NULL == this ) return NULL;
		return _renderer;
	}
	
	inline Audio* GetAudio()
	{
		if ( NULL == this ) return NULL;
		return _audio;
	}
	
	inline Physics* GetPhysics()
	{
		if ( NULL == this ) return NULL;
		return _physics;
	}
	
	inline Environment* GetEnvironment()
	{
		if ( NULL == this ) return NULL;
		return _environment;
	}
	
	inline Features* GetFeatures()
	{
		if ( NULL == this ) return NULL;
		return _features;
	}
	
	inline Actors* GetActors()
	{
		if ( NULL == this ) return NULL;
		return _actors;
	}
	
	inline Ego* GetEgo()
	{
		if ( NULL == this ) return NULL;
		return _ego;
	}
	
	inline scalar GetTime() const
	{
		return _time;
	}
	
	inline scalar GetDelta() const
	{
		return _delta;
	}
	
	inline void SetDepthQuery( int x, int y )
	{
		_queryPixel.x = (scalar)x;
		_queryPixel.y = (scalar)y;
	}
	
	inline vec3 GetDepthQuery()
	{
		return _queryResult;
	}
	
// Operations
public:
	bool			Create( PCSTR path );
	bool			Load( PCSTR path );
	bool			Save();
public:
	bool			Start();
	void			Stop();
public:
	bool			PrepareContext();
	void			Update( scalar time, const vec3& direction, UINT actions );
	void			Render();
	
// Data
protected:
	static Application* _instance;
protected:
	string			_path;
	XMLElement*		_root;
protected:
	Renderer*		_renderer;
	Audio*			_audio;
	Physics*		_physics;
protected:
	Environment*	_environment;
	Features*		_features;
	Actors*			_actors;
	Ego*			_ego;
	Guide*			_guide;
protected:
	scalar			_time;
	scalar			_delta;
protected:
	vec2			_queryPixel;
	vec3			_queryResult;
};
