//
// Audio.h
// Copyright (c) Michael J Stokes, 2007.  All rights reserved.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#pragma once

class Sound;
class Emitter;


class Audio
{
// Construction
public:
	Audio();
	virtual ~Audio();
	
// Inlines
public:
	inline bool IsActive() const
	{
		return _active;
	}
	
// Operations
public:
	virtual bool		Initialize();
	virtual void		Terminate();
	virtual void		Lock() = 0;
	virtual void		Unlock() = 0;
	virtual void		Pause( bool pause ) = 0;
	virtual void		Synchronize();
public:
	virtual Emitter*	Register();
	virtual void		Delete( Emitter* emitter );
	virtual void		SetListener( const vec3& position, const vec3& forward, const vec3& right );
public:
	Sound*				LoadSound( PCSTR name );
	
// Implementation
protected:
	virtual void		Process( int* output, UINT samples );
	
// Source
protected:
	struct Source
	{
		~Source();
		Emitter*		emitter;
		void*			sound;
		UINT			channel;
		bool			loop;
		int				volume;
	};
	
// Data
protected:
	bool				_active;
	list<Emitter*>		_emitters;
	list<Source*>		_sources;
	list<Source*>		_cleanup;
	list<Sound*>		_sounds;
	
	friend class Emitter;
};


class Emitter
{
// Construction
protected:
	inline			Emitter( Audio* audio ) { _audio = audio; _left = _right = 0; }
	inline			~Emitter() { Silence(); }
	
// Operations
public:
	void			Play( Sound* sound, UINT channel = 1, bool loop = false, scalar volume = 1 );
	void			Silence( UINT channels = 0xFFFFFFFF );
	bool			IsPlaying( UINT channels = 0xFFFFFFFF );
	inline void		SetPosition( const vec3& position ) { _position = position; }
	
// Data
protected:
	Audio*					_audio;
	list<Audio::Source*>	_sources;
	vec3					_position;
	UINT					_left;
	UINT					_right;
	
	friend class Audio;
	friend struct Audio::Source;
};
