//
// Buffer.cpp
// Copyright Michael J Stokes, 2008.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "Common.h"
#include "Buffer.h"

#define TEMP_BUFFER			4096
#define BLOCK_SIZE			1024
#define BLOCK_MASK			0xFFFFFC00


///////////////////////////////////////////////////////////////////////////////
// Buffer allocate space

void * Buffer::Allocate( UINT length, bool add )
{
	if ( _allocated - _length < length )
	{
		_allocated	= _length + length;
		_allocated	= ( _allocated + BLOCK_SIZE - 1 ) & BLOCK_MASK;
		_base		= (BYTE*)realloc( _base, _allocated );
	}
	
	if ( add )
	{
		void * ret = _base + _length;
		_length += length;
		return ret;
	}
	else
	{
		return _base + _length;
	}
}

///////////////////////////////////////////////////////////////////////////////
// Buffer socket transmit

void Buffer::Transmit( SOCKET sock )
{
	while ( _length > 0 )
	{
#ifdef _WIN32
		int length = send( sock, (PCSTR)_base, _length, 0 );
#else
		int length = send( sock, (PCSTR)_base, _length, MSG_NOSIGNAL );
#endif
		if ( length <= 0 ) break;
		Remove( length );
	}
}

///////////////////////////////////////////////////////////////////////////////
// Buffer socket receive

bool Buffer::Receive( SOCKET sock )
{
	BYTE buffer[ TEMP_BUFFER ];
	
	for ( ; ; )
	{
		int length = recv( sock, (PSTR)buffer, TEMP_BUFFER, 0 );
		if ( length == 0 ) return false;
		if ( length < 0 ) break;
		Add( buffer, length );
	}
	
	return true;
}
