//
// Buffer.h
// Copyright Michael J Stokes, 2008.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#pragma once


class Buffer
{
// Construction
public:
	inline Buffer()
	{
		_base		= NULL;
		_length		= 0;
		_allocated	= 0;
	}
	
	inline ~Buffer()
	{
		if ( _base ) free( _base );
	}
	
// Operations
public:
	void*	Allocate( UINT length, bool add = false );
public:
	void	Transmit( SOCKET sock );
	bool	Receive( SOCKET sock );
	
// Inline Operations
public:
	inline void Empty()
	{
		_length = 0;
	}
	
	inline bool IsEmpty() const
	{
		return ( 0 == _length );
	}
	
	inline BYTE* GetPointer()
	{
		return _base;
	}
	
	inline const BYTE* GetPointer() const
	{
		return _base;
	}
	
	inline UINT GetLength() const
	{
		return _length;
	}
	
	inline void SetLength( UINT length )
	{
		_length = ( _allocated < length ) ? _allocated : length;
	}
	
	inline void AddLength( int add )
	{
		SetLength( _length + add );
	}
	
	inline UINT GetBufferSize() const
	{
		return _allocated;
	}
	
	inline void Add( const void * data, UINT length )
	{
		if ( _length + length > _allocated )
		{
			Allocate( length );
		}
		
		CopyMemory( _base + _length, data, length );
		_length += length;
	}
	
	inline void Add( const Buffer& buffer )
	{
		Add( buffer._base, buffer._length );
	}
	
	inline void Insert( UINT offset, const void * data, UINT length )
	{
		if ( _length + length > _allocated )
		{
			Allocate( length );
		}
		
		MoveMemory( _base + offset + length, _base + offset, _length - offset );
		CopyMemory( _base + offset, data, length );
		_length += length;
	}
	
	inline void Remove( UINT length )
	{
		if ( length == 0 ) return;
		if ( length > _length ) return;
		_length -= length;
		MoveMemory( _base, _base + length, _length );
	}
	
	inline void Print( PCSTR psz )
	{
		Add( psz, (UINT)strlen( psz ) );
	}
	
// Data
protected:
	BYTE*	_base;
	UINT	_length;
	UINT	_allocated;
};
