//
// Common.h
// Copyright Michael J Stokes, 2008.
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#pragma once


// Platform

#define _CRT_SECURE_NO_WARNINGS

#include <math.h>
#include <string>
#include <iostream>
#include <sstream>
#include <iomanip>
#include <fstream>
#include <vector>
#include <list>
#include <map>
#include <assert.h>

#ifdef _WIN32
#define NOMINMAX
#include <windows.h>
#include <winsock.h>
#else
#include <sys/stat.h>
#endif


// VRJuggler

#ifdef WITH_VRJUGGLER

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glext.h>

#pragma warning (disable:4275)
#pragma warning (disable:4251)
#include <vrj/vrjConfig.h>
#include <vrj/Kernel/Kernel.h>
#include <vrj/Draw/OGL/GlApp.h>
#include <gadget/Type/PositionInterface.h>
#include <gadget/Type/DigitalInterface.h>

#endif


// SDL

#ifdef WITH_SDL

#include <SDL.h>
#include <SDL_opengl.h>

#endif


// PhysX

#ifdef _WIN32
#include <crtdbg.h>
#endif

#include <Nx.h>
#include <NxPhysics.h>
#include <NxCooking.h>
#include <NxStream.h>
#include <NxControllerManager.h>
#include <NxCapsuleController.h>
#include <NxUserAllocatorDefault.h>


// wxWidgets

#ifdef IS_EDITOR
#define WITH_WXWIDGETS

#ifdef _DEBUG
#define __WXDEBUG__
#define WXUSINGDLL 1
#endif

#include <wx/wx.h>
#include <wx/file.h>
#include <wx/dnd.h>
#include <wx/utils.h>
#include <wx/config.h>
#include <wx/settings.h>
#include <wx/filename.h>
#include <wx/stdpaths.h>
#include <wx/mimetype.h>
#include <wx/dirdlg.h>
#include <wx/treectrl.h>
#include <wx/listctrl.h>
#include <wx/splitter.h>
#include <wx/wupdlock.h>
#include <wx/spinctrl.h>
#include <wx/dcbuffer.h>
#include <wx/statline.h>
#include <wx/imaglist.h>
#include <wx/aboutdlg.h>
#include <wx/artprov.h>
#include <wx/glcanvas.h>
#include <wx/gbsizer.h>
#include <wx/tokenzr.h>
#include <wx/richtext/richtextctrl.h>

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glext.h>

#define T(x) x

#endif


// Types

#ifdef _WIN32
#define PATH_SLASH '\\'
#define PATH_SLASH_STR "\\"
#else
#define PATH_SLASH '/'
#define PATH_SLASH_STR "/"
#endif

typedef signed char				INT8;
typedef unsigned char			UINT8, BYTE;
typedef signed short			INT16;
typedef unsigned short			UINT16;
typedef signed int				INT32, INT;
typedef unsigned int			UINT32, UINT;

typedef char *					PSTR;
typedef const char *			PCSTR;


// Import some core functionality from STL

using std::string;
using std::cout;
using std::cerr;
using std::endl;
using std::vector;
using std::list;
using std::map;
using std::multimap;


// Local

#include "RCO.h"
#include "3DMath.h"
#include "XML.h"
