//
// Ego.cpp
// Copyright Michael J Stokes, 2008.
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "Common.h"
#include "Ego.h"


////////////////////////////////////////////////////////////////////////////////
// Ego - construction

Ego::Ego()
{
	_position.y = 20;
}

////////////////////////////////////////////////////////////////////////////////
// Ego - destruction

Ego::~Ego()
{
}

////////////////////////////////////////////////////////////////////////////////
// Ego - load

bool Ego::Load( XMLElement* root )
{
	if ( XMLElement* ego = root->GetElement( "ego" ) )
	{
		if ( XMLElement* position = ego->GetElement( "position" ) )
		{
			vec3 pos;
			pos.x = position->GetAttributeFloat( "x" );
			pos.y = position->GetAttributeFloat( "y" );
			pos.z = position->GetAttributeFloat( "z" );
			SetPosition( pos );
		}
	}
	
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// Ego - set the position

void Ego::SetPosition( const vec3& position )
{
	_position = position;
}

////////////////////////////////////////////////////////////////////////////////
// Ego - update

void Ego::Update( const vec3& direction, UINT actions )
{
	_direction = direction;
}

////////////////////////////////////////////////////////////////////////////////
// Ego - pre-render

void Ego::PreRender()
{
	glTranslatef( -_position.x, -_position.y, -_position.z );
	
	vec3 dir( -_direction.x, 1, -_direction.z );
	dir.Normalize();
	scalar v[ 4 ] = { dir.x, dir.y, dir.z, 0 };
	glLightfv( GL_LIGHT0, GL_POSITION, v );
}
