//
// EnvironmentMesh.cpp
// Copyright Michael J Stokes, 2008.
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "Common.h"
#include "Application.h"
#include "Renderer.h"
#include "Physics.h"
#include "Mesh.h"
#include "EnvironmentMesh.h"


////////////////////////////////////////////////////////////////////////////////
// EnvironmentMesh construction

EnvironmentMesh::EnvironmentMesh()
{
	_scale		= 0;
	_flags		= NONE;
	
	_mesh		= NULL;
	_physics	= NULL;
}

////////////////////////////////////////////////////////////////////////////////
// EnvironmentMesh destruction

EnvironmentMesh::~EnvironmentMesh()
{
	if ( NULL != _mesh )
	{
		_mesh->Release();
	}
	
	if ( NULL != _physics )
	{
		Application::GetInstance()->GetPhysics()->RemoveMesh( _physics );
	}
}

////////////////////////////////////////////////////////////////////////////////
// EnvironmentMesh load

bool EnvironmentMesh::Load( XMLElement* xml )
{
	PCSTR name = xml->GetAttributeString( "source" );
	if ( NULL == name ) return false;
	_name = name;
	
	string textures( Application::GetInstance()->GetPath() + "/environment/textures/" );
	string obj( Application::GetInstance()->GetPath() + "/environment/meshes/" );
	obj += name;
	
	_scale = xml->GetAttributeFloat( "scale", 1 );
	
	_mesh = Application::GetInstance()->GetRenderer()->LoadMesh( obj.c_str(), textures.c_str(), _scale );
	if ( NULL == _mesh ) return false;
	
	if ( xml->GetElement( "clip" ) ) _flags |= CLIP;
	if ( xml->GetElement( "render" ) ) _flags |= RENDER;
	if ( xml->GetElement( "depth" ) ) _flags |= DEPTH;
	
	if ( _flags & CLIP )
	{
		_physics = Application::GetInstance()->GetPhysics()->AddMesh( _mesh );
	}
	
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// EnvironmentMesh save

XMLElement* EnvironmentMesh::Save()
{
	XMLElement* xml = new XMLElement( "mesh" );
	
	xml->AddAttribute( "source", _name.c_str() );
	xml->AddAttributeFloat( "scale", _scale );
	
	if ( _flags & CLIP ) xml->AddElement( "clip" );
	if ( _flags & RENDER ) xml->AddElement( "render" );
	if ( _flags & DEPTH ) xml->AddElement( "depth" );
	
	return xml;
}

////////////////////////////////////////////////////////////////////////////////
// EnvironmentMesh render

void EnvironmentMesh::Render()
{
	if ( NULL != _mesh )
	{
		glDisable( GL_LIGHTING );
		
		if ( _flags & RENDER )
		{
			_mesh->Render();
		}
		else if ( _flags & DEPTH )
		{
			glColorMask( GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE );
			_mesh->Render();
			glColorMask( GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE );
		}
		
		glEnable( GL_LIGHTING );
	}
}
