//
// Feature.h
// Copyright Michael J Stokes, 2008.
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#pragma once


class Feature
{
// Construction
public:
	Feature( PCSTR name );
	~Feature();
	
// ActivationPoint
public:
	struct ActivationPoint
	{
		vec3	position;
		scalar	inner;
		scalar	outer;
	};
	
// Inlines
public:
	const string & GetName() const
	{
		return _name;
	}
	
	inline const vector<ActivationPoint>& GetActivations() const
	{
		return _activations;
	}
	
	inline const vector<vec3>& GetLocations() const
	{
		return _locations;
	}
	
	inline const vector<vec3>& GetGuides() const
	{
		return _guides;
	}
	
	inline scalar GetPresented() const
	{
		return _presented;
	}
	
	inline scalar GetSuggested() const
	{
		return _suggested;
	}
	
	inline XMLElement* GetScript()
	{
		return _script;
	}
	
// Operations
public:
	bool		Load();
	bool		Save();
	bool		Rename( PCSTR name );
	bool		Delete( PCSTR path = NULL );
public:
	void		RegisterPositions();
	bool		IsInsideActivationPoint( const vec3& position );
	bool		IsOutsideActivationPoints( const vec3& position );
	string		GetClosestGuidePoint( const vec3& position );
	string		GetClosestLocationPoint( const vec3& position );
	XMLElement*	Present();
	void		Suggest();
public:
	int			AddPoint( int type, const vec3& position );
	bool		RemovePoint( int type, int index );
	bool		MovePoint( int type, int index, const vec3& position );
	bool		CheckPoint( int type, int index );
	bool		GetActivationRadii( int index, scalar& inner, scalar& outer, vec3& position );
	bool		SetActivationRadii( int index, scalar inner, scalar outer );
	void		SetScript( XMLElement* script );
public:
	void		Render();
	bool		HitTest( const vec3& origin, const vec3& direction, scalar& distance, int& type, int& index, bool selected ) const;
	
// Implementation
public:
	static void	RenderBox( const vec3& center, const vec3& colour, bool selected, scalar size = 0.15f );
	static void	RenderFloatingIcon( const ActivationPoint& ap, bool selected );
	static bool	RayTest( const vec3& origin, const vec3& direction, scalar& distance, const vec3& center, scalar size = 0.25f );
	
// Data
protected:
	string					_name;
	vector<ActivationPoint>	_activations;
	vector<vec3>			_locations;
	vector<vec3>			_guides;
	XMLElement*				_script;
protected:
	scalar					_presented;
	scalar					_suggested;
};
