//
// Features.h
// Copyright Michael J Stokes, 2008.
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#pragma once

class Feature;


class Features
{
// Construction
public:
	Features();
	~Features();
	
// Selection
public:
	struct Selection
	{
		Feature*	feature;
		int			type;
		int			index;
		
		inline Selection() { feature = NULL; }
		
		inline bool operator == ( const Selection& o ) const
		{
			return ( feature == o.feature ) && ( type == o.type ) && ( index == o.index );
		}
		
		inline bool operator != ( const Selection& o ) const
		{
			return ( feature != o.feature ) || ( type != o.type ) || ( index != o.index );
		}
	};
	
// Inlines
public:
	const list<Feature*> & GetFeatures() const
	{
		return _features;
	}
	
	inline Selection& GetSelection()
	{
		return _selection;
	}
	
// Operations
public:
	bool			Load();
	void			Render();
	void			RegisterPositions();
	Feature*		GetActivatedFeature( const vec3& position );
	Feature*		SelectFeature( const vec3& position );
	Selection		HitTest( const vec3& origin, const vec3& ray, scalar& distance );
	Feature*		AddFeature();
	void			DeleteFeature( Feature* feature );
	
// Data
protected:
	list<Feature*>	_features;
	Selection		_selection;
};
