//
// Guide.h
// Copyright Michael J Stokes, 2008.
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#pragma once

class Feature;


class Guide
{
// Construction
public:
	Guide();
	virtual ~Guide();
	
// Operations
public:
	bool			Load( XMLElement* root );
	void			Update();
	
// State
protected:
	enum GuideState
	{
		GS_FOLLOWING, GS_WAITING_TO_FOLLOW,
		GS_LEADING, GS_WAITING_TO_LEAD, GS_WAITING_TO_PRESENT,
		GS_SUGGESTING, GS_WALKING_TO_PRESENT, GS_PRESENTING
	} _state;
	
// Implementation
protected:
	void			StartFollowing();
	void			RunFollowing();
	void			StartWaitingToFollow( scalar delay = 5.0f );
	void			RunWaitingToFollow();
	void			StartLeading();
	void			RunLeading();
	void			StartWaitingToLeadOrPresent( bool lead );
	void			RunWaitingToLeadOrPresent( bool lead );
	void			StartSuggesting();
	void			RunSuggesting();
	void			StartWalkingToPresent();
	void			RunWalkingToPresent();
	void			StartPresenting();
	void			RunPresenting();
protected:
	XMLElement*		Send( XMLElement* request );
	void			SetState( GuideState state );
	bool			CheckActivation();
	
// Data
protected:
	scalar			_now;
	Ego*			_ego;
	vec3			_position;
	vec3			_toward;
	scalar			_distance;
	scalar			_transition;
	scalar			_delay;
	Feature*		_target;
	Feature*		_ignore;
	vec3			_base;
	XMLElement*		_script;
};
