//
// Mesh.h
// Copyright Michael J Stokes, 2008.
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#pragma once

#include "Renderer.h"

class Material;


class Mesh : public Resource
{
// Construction
public:
	Mesh( PCSTR name, PCSTR materialPath, scalar scale = 1 );
	virtual ~Mesh();
	
// Operations
public:
	virtual bool	Load();
	virtual bool	Upload();
	virtual void	Erase();
	virtual void	Render();
	
// Vertex
protected:
	struct Vertex
	{
		vec3 position;
		vec3 normal;
		vec2 material;
	};
	
// Group
protected:
	struct Group
	{
		Material*	material;
		UINT32		firstIndex;
		UINT32		indexCount;
	};
	
// Data
protected:
	string			_materials;
	scalar			_scale;
	vector<Vertex>	_vertices;
	vector<UINT32>	_indices;
	vector<Group>	_groups;
	GLuint			_vbuf;
	GLuint			_ibuf;
	
	friend class Physics;
};
