//
// Physics.h
// Copyright Michael J Stokes, 2008.
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#pragma once

class Mesh;


class Physics
{
// Construction
public:
	Physics();
	~Physics();
	
// Inlines
public:
	inline NxPhysicsSDK* GetSDK()
	{
		return _sdk;
	}
	
	inline NxScene* GetScene()
	{
		return _scene;
	}
	
	inline NxControllerManager* GetControllerManager()
	{
		return _manager;
	}
	
// Operations
public:
	void			Start( scalar delta );
	void			Finish();
	void*			AddMesh( Mesh* mesh );
	void			RemoveMesh( void* handle );
	scalar			Drop( const vec3& position );
	
// WriteStream
public:
	class ReadStream;
	
	class WriteStream : public NxStream
	{
	public:
		WriteStream();
		virtual ~WriteStream();
	public:
		virtual NxU8		readByte() const { return 0; }
		virtual NxU16		readWord() const { return 0; }
		virtual NxU32		readDword() const { return 0; }
		virtual float		readFloat() const { return 0; }
		virtual double		readDouble() const { return 0; }
		virtual void		readBuffer( void* buffer, NxU32 size ) const {};
		virtual NxStream&	storeByte( NxU8 b );
		virtual NxStream&	storeWord( NxU16 w );
		virtual NxStream&	storeDword( NxU32 d );
		virtual NxStream&	storeFloat( NxReal f );
		virtual NxStream&	storeDouble( NxF64 f );
		virtual NxStream&	storeBuffer( const void* buffer, NxU32 size );
	protected:
		BYTE*				_data;
		UINT				_length;
		friend class		Physics::ReadStream;
	};
	
// ReadStream
public:
	class ReadStream : public NxStream
	{
	public:
		ReadStream( WriteStream& source );
		ReadStream( const void* source, UINT length );
		ReadStream( const ReadStream& copy );
		virtual ~ReadStream();
	public:
		virtual NxU8		readByte() const;
		virtual NxU16		readWord() const;
		virtual NxU32		readDword() const;
		virtual float		readFloat() const;
		virtual double		readDouble() const;
		virtual void		readBuffer( void* buffer, NxU32 size ) const;
		virtual NxStream&	storeByte( NxU8 b ) { return *this; }
		virtual NxStream&	storeWord( NxU16 w ) { return *this; }
		virtual NxStream&	storeDword( NxU32 d ) { return *this; }
		virtual NxStream&	storeFloat( NxReal f ) { return *this; }
		virtual NxStream&	storeDouble( NxF64 f ) { return *this; }
		virtual NxStream&	storeBuffer( const void* buffer, NxU32 size ) { return *this; }
	protected:
		class Internal
		{
		public:
			Internal( const void* source, UINT length );
			void ref();
			void unref();
		public:
			const BYTE*		_data;
			UINT			_length;
			UINT			_pos;
			UINT			_refs;
		} *_internal;
	};
	
// Data
protected:
	NxPhysicsSDK*			_sdk;
	NxScene*				_scene;
	NxControllerManager*	_manager;
};
