//
// PhysicsEgo.cpp
// Copyright Michael J Stokes, 2008.
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "Common.h"
#include "Application.h"
#include "Physics.h"
#include "PhysicsEgo.h"


////////////////////////////////////////////////////////////////////////////////
// PhysicsEgo - construction

PhysicsEgo::PhysicsEgo()
{
	_acceleration		= 30;
	_friction			= 3;
	_gravity			= -20;
	_radius				= 0.15f;
	_height				= 1.2f;
	
	NxCapsuleControllerDesc desc;
	
	desc.radius			= _radius;
	desc.height			= _height;
	desc.stepOffset		= 0.6f;
	desc.position.x		= (scalar)_position.x;
	desc.position.y		= (scalar)_position.y - _radius * 5;
	desc.position.z		= (scalar)_position.z;
	
	Physics* physics	= Application::GetInstance()->GetPhysics();
	_controller			= physics->GetControllerManager()->createController( physics->GetScene(), desc );
}

////////////////////////////////////////////////////////////////////////////////
// PhysicsEgo - destruction

PhysicsEgo::~PhysicsEgo()
{
	Application::GetInstance()->GetPhysics()->GetControllerManager()->releaseController( *_controller );
}

////////////////////////////////////////////////////////////////////////////////
// PhysicsEgo - set the position

void PhysicsEgo::SetPosition( const vec3& position )
{
	_position = position;
	NxExtendedVec3 pos( _position.x, _position.y + _radius * 5, _position.z );
	_controller->setPosition( pos );
}

////////////////////////////////////////////////////////////////////////////////
// PhysicsEgo - update

void PhysicsEgo::Update( const vec3& direction, UINT actions )
{
	Ego::Update( direction, actions );
	
	scalar delta = Application::GetInstance()->GetDelta();
	if ( delta <= 0 ) return;
	
	scalar request = 0;
	if ( actions & Application::ACTION_FORWARD ) request += 1;
	if ( actions & Application::ACTION_BACKWARD ) request -= 1;
	
	_velocity += direction * ( _acceleration * delta * request );
	_velocity.y += _gravity * delta;
	
	vec3 scaled( _velocity * delta );
	
	NxU32 flags = 0;
	NxVec3 nxScaled( scaled.x, scaled.y, scaled.z );
	_controller->move( nxScaled, 0xFFFFFFFF, 0, flags );
	
	const NxExtendedVec3& nxPos( _controller->getDebugPosition() );
	vec3 pos( (scalar)nxPos.x, (scalar)nxPos.y - _radius * 5, (scalar)nxPos.z );
	
	scaled = ( ( pos - _position ) / delta );
	if ( scaled.y > _velocity.y ) scaled.y = _velocity.y;
	_velocity = scaled;
	_position = pos;
	
	scaled = _velocity;
	scaled.y = 0;
	_velocity -= scaled * _friction * delta;
}
