//
// Renderer.h
// Copyright Michael J Stokes, 2008.
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#pragma once


extern PFNGLGENBUFFERSPROC					glGenBuffers;
extern PFNGLDELETEBUFFERSPROC				glDeleteBuffers;
extern PFNGLBINDBUFFERPROC					glBindBuffer;
extern PFNGLBUFFERDATAPROC					glBufferData;

extern PFNGLCREATESHADERPROC				glCreateShader;
extern PFNGLSHADERSOURCEPROC				glShaderSource;
extern PFNGLCOMPILESHADERPROC				glCompileShader;
extern PFNGLGETSHADERIVPROC					glGetShaderiv;
extern PFNGLGETSHADERINFOLOGPROC			glGetShaderInfoLog;
extern PFNGLATTACHSHADERPROC				glAttachShader;
extern PFNGLDETACHSHADERPROC				glDetachShader;
extern PFNGLDELETESHADERPROC				glDeleteShader;
extern PFNGLCREATEPROGRAMPROC				glCreateProgram;
extern PFNGLLINKPROGRAMPROC					glLinkProgram;
extern PFNGLVALIDATEPROGRAMPROC				glValidateProgram;
extern PFNGLGETPROGRAMIVPROC				glGetProgramiv;
extern PFNGLGETPROGRAMINFOLOGPROC			glGetProgramInfoLog;
extern PFNGLUSEPROGRAMPROC					glUseProgram;
extern PFNGLDELETEPROGRAMPROC				glDeleteProgram;
extern PFNGLGETUNIFORMLOCATIONPROC			glGetUniformLocation;
extern PFNGLUNIFORM1IPROC					glUniform1i;
extern PFNGLUNIFORM1FPROC					glUniform1f;
extern PFNGLUNIFORM3FVPROC					glUniform3fv;
extern PFNGLUNIFORM4FVPROC					glUniform4fv;
extern PFNGLUNIFORMMATRIX3FVPROC			glUniformMatrix3fv;
extern PFNGLUNIFORMMATRIX4FVPROC			glUniformMatrix4fv;
extern PFNGLBINDATTRIBLOCATIONPROC			glBindAttribLocation;
extern PFNGLVERTEXATTRIBPOINTERPROC			glVertexAttribPointer;
extern PFNGLENABLEVERTEXATTRIBARRAYPROC		glEnableVertexAttribArray;
extern PFNGLDISABLEVERTEXATTRIBARRAYPROC	glDisableVertexAttribArray;

class Material;
class Shader;
class Mesh;
class Character;


class Renderer
{
// Construction
public:
	Renderer();
	~Renderer();
	
// Inline
public:
	inline Shader* GetSkeletonShader()
	{
		return _skeletonShader;
	}
	
	inline Mesh* GetInfoMesh()
	{
		return _infoMesh;
	}
	
// Operations
public:
	bool			Initialize();
	bool			PrepareContext();
public:
	Shader*			LoadShader( PCSTR name );
	Material*		LoadMaterial( PCSTR name );
	Mesh*			LoadMesh( PCSTR name, PCSTR materialPath, scalar scale = 1 );
	Character*		LoadCharacter( PCSTR name );
public:
	static void		RenderCylinder( scalar radius, scalar height );
	
// Implementation
protected:
	void*			GetGLProc( PCSTR name );
	
// Data
protected:
	list<Shader*>		_shaders;
	list<Material*>		_materials;
	list<Mesh*>			_meshes;
	list<Character*>	_characters;
protected:
	Shader*				_skeletonShader;
	Mesh*				_infoMesh;
};


class Resource : public NamedObject
{
// Construction
protected:
	inline Resource( PCSTR name ) : NamedObject( name ) {};
	virtual ~Resource() {};
	
// Operations
public:
	virtual bool	Load() { return false; }
	virtual bool	Upload() { return false; }
	virtual void	Erase() {};
	virtual void	Render() {};
};
