//
// Shader.h
// Copyright Michael J Stokes, 2008.
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#pragma once

#include "Renderer.h"


class Shader : public Resource
{
// Construction
public:
	Shader( PCSTR name );
	virtual ~Shader();
	
// Constants
public:
	enum
	{
		BONE_INDEX_ATTRIB	= 5,
		BONE_WEIGHT_ATTRIB	= 6,
	};
	
// Operations
public:
	virtual bool	Load();
	virtual bool	Upload();
	virtual void	Erase();
public:
	void			Begin();
	void			End();
	int				FindUniform( PCSTR name );
public:
	void			SetBoneMatrices( const matrix4x4* transform, int count );
	
// Implementation
protected:
	string			Load( const string & name );
	GLuint			Compile( GLenum type, const string & source );
	
// Data
protected:
	string			_vertex;
	string			_fragment;
protected:
	int				_boneMatrix;
protected:
#ifdef WITH_VRJUGGLER
	GlContextData<GLuint> _program;
#else
	GLuint			_program;
#endif
protected:
	static PCSTR	INTERNAL[];
};
