//
// Sound.h
// Copyright Michael J Stokes, 2008.
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#pragma once


class Sound : public RCO
{
// Construction
public:
	Sound( PCSTR name );
	virtual ~Sound();
	
	class Instance;
	
// Inlines
public:
	inline bool IsNamed( PCSTR name ) const
	{
		return ( _name == name );
	}
	
// Operations
public:
	virtual bool		Load();
	virtual Instance*	CreateInstance();
	
// Implementation
protected:
	bool				LoadWave();
	bool				LoadVorbis();
	
// Instance
public:
	class Instance
	{
	public:
		Instance( Sound* sound );
		virtual ~Instance();
	public:
		virtual bool Mix( int* output, UINT samples, int left, int right, bool loop ) = 0;
	protected:
		Sound*	_sound;
	};
	
// WaveInstance
public:
	class WaveInstance : public Instance
	{
	public:
		WaveInstance( Sound* sound );
	public:
		virtual bool Mix( int* output, UINT samples, int left, int right, bool loop );
	protected:
		UINT	_position;
	};
	
// VorbisInstance
public:
	class VorbisInstance : public Instance
	{
	public:
		VorbisInstance( Sound* sound );
		virtual ~VorbisInstance();
	public:
		virtual bool Mix( int* output, UINT samples, int left, int right, bool loop );
	protected:
		void*	_handle;
	};
	
// Data
protected:
	string		_name;
	UINT		_duration;
	short*		_samples;
	bool		_vorbis;
};
