//
// BMLFrame.cpp
// Copyright Michael J Stokes, 2009.
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "Common.h"
#include "Editor.h"
#include "Application.h"
#include "Actors.h"
#include "Actor.h"
#include "BMLFrame.h"


IMPLEMENT_ABSTRACT_CLASS( BMLFrame, wxFrame )

BEGIN_EVENT_TABLE( BMLFrame, wxFrame )
	EVT_CLOSE( BMLFrame::OnClose )
	EVT_ACTIVATE( BMLFrame::OnActivate )
	EVT_CHAR_HOOK( BMLFrame::OnCharHook )
	EVT_BUTTON( ID_BML_EXECUTE, BMLFrame::OnExecute )
	EVT_BUTTON( ID_BML_LAST, BMLFrame::OnLast )
	EVT_COMBOBOX( 3, BMLFrame::OnTemplate )
END_EVENT_TABLE()

BMLFrame* BMLFrame::_instance = NULL;


//////////////////////////////////////////////////////////////////////////////
// BMLFrame construction

BMLFrame::BMLFrame() : wxFrame( NULL, -1, T("BML Console") )
{
	_splitter	= new wxSplitterWindow( this, wxID_ANY, wxDefaultPosition, wxSize( 0, 0 ) );
	_view		= new wxRichTextCtrl( _splitter, 1, wxEmptyString, wxDefaultPosition, wxSize( 0, 0 ), wxVSCROLL | wxNO_BORDER | wxWANTS_CHARS );
	_panel		= new wxPanel( _splitter, wxID_ANY );
	
	_splitter->SetMinimumPaneSize( 32 );
	_splitter->SetSashGravity( 1 );
	_splitter->SplitHorizontally( _view, _panel, -200 );
	
	wxBoxSizer* root = new wxBoxSizer( wxVERTICAL );
	wxGridBagSizer* top = new wxGridBagSizer( 0, 6 );
	top->Add( new wxStaticText( _panel, wxID_ANY, "Actor:" ), wxGBPosition( 0, 0 ), wxDefaultSpan, wxALIGN_CENTRE_VERTICAL );
	top->Add( _actor = new wxComboBox( _panel, 2, "", wxDefaultPosition, wxDefaultSize, 0, NULL, wxCB_SORT ), wxGBPosition( 0, 1 ), wxDefaultSpan, wxALIGN_CENTRE_VERTICAL | wxEXPAND );
	top->Add( new wxStaticText( _panel, wxID_ANY, "Template:" ), wxGBPosition( 0, 2 ), wxDefaultSpan, wxALIGN_CENTRE_VERTICAL );
	top->Add( _template = new wxComboBox( _panel, 3, "", wxDefaultPosition, wxDefaultSize, 0, NULL, wxCB_READONLY ), wxGBPosition( 0, 3 ), wxDefaultSpan, wxALIGN_CENTRE_VERTICAL | wxEXPAND );
	top->Add( new wxButton( _panel, ID_BML_EXECUTE, "Execute" ), wxGBPosition( 0, 4 ), wxDefaultSpan, wxALIGN_CENTRE_VERTICAL );
	top->Add( new wxButton( _panel, ID_BML_LAST, "Last" ), wxGBPosition( 0, 5 ), wxDefaultSpan, wxALIGN_CENTRE_VERTICAL );
	top->AddGrowableCol( 1 );
	top->AddGrowableCol( 3 );
	root->Add( top, 0, wxEXPAND | wxALL, 8 );
	root->Add( _edit = new wxRichTextCtrl( _panel, 1, wxEmptyString, wxDefaultPosition, wxSize( 0, 0 ), wxVSCROLL | wxWANTS_CHARS ), 1, wxEXPAND );
	_panel->SetSizer( root );
	
	_view->SetEditable( false );
	_view->SetFont( wxFont( wxSystemSettings::GetFont( wxSYS_ANSI_FIXED_FONT ) ) );
	_edit->SetFont( wxFont( wxSystemSettings::GetFont( wxSYS_ANSI_FIXED_FONT ) ) );
	
	BuildActorList();
	
	_template->Append( "<vgs:create>" );
	_templates.push_back( "<vgs:create>\n <character model=\"brad\"/>\n <position target=\"wall_a1\"/>\n <orientation heading=\"90\"/>\n</vgs:create>" );
	_template->Append( "<vgs:destroy>" );
	_templates.push_back( "<vgs:destroy/>" );
	_template->Append( "<head type=\"NOD\">" );
	_templates.push_back( "<bml>\n <head type=\"NOD\" repeats=\"1.0\" amount=\"0.5\" velocity=\"1.0\"/>\n</bml>" );
	_template->Append( "<head type=\"SHAKE\">" );
	_templates.push_back( "<bml>\n <head type=\"SHAKE\" repeats=\"1.0\" amount=\"0.5\" velocity=\"1.0\"/>\n</bml>" );
	_template->Append( "<body posture>" );
	_templates.push_back( "<bml>\n <body posture=\"LHandOnHip_Motex\"/>\n</bml>" );
	_template->Append( "<gaze>" );
	_templates.push_back( "<bml>\n <gaze target=\"ego\"/>\n</bml>" );
	_template->Append( "<sbm:animation>" );
	_templates.push_back( "<bml>\n <sbm:animation name=\"HandsAtSide_RArm_GestureYou\"/>\n</bml>" );
	_template->Append( "<vgs:walk>" );
	_templates.push_back( "<bml>\n <vgs:walk target=\"ego\" proximity=\"2.0\"/>\n</bml>" );
	_template->Append( "<vgs:speak>" );
	_templates.push_back( "<bml>\n <vgs:speak file=\"follow.ogg\"/>\n</bml>" );
	
	SetSize( wxSize( 600, 600 ) );
	SetMinSize( wxSize( 300, 300 ) );
	CenterOnScreen();
	
	_edit->SetFocus();
	
	_instance = this;
}

//////////////////////////////////////////////////////////////////////////////
// BMLFrame destruction

BMLFrame::~BMLFrame()
{
	_instance = NULL;
}

//////////////////////////////////////////////////////////////////////////////
// BMLFrame add some XML

void BMLFrame::AddXML( PCSTR actor, XMLElement* xml )
{
	if ( xml->IsNamed( "vgs", "position" ) || xml->IsNamed( "vgs", "walking" ) || xml->IsNamed( "vgs", "speaking" ) ) return;
	
	long pos = _view->GetLastPosition();
	_view->SetInsertionPoint( pos );
	_view->SetSelection( pos, pos );
	
	_view->EndAllStyles();
	_view->BeginBold();
	_view->WriteText( wxString( actor ) + ":" );
	_view->EndBold();
	_view->Newline();
	
	AddXML( xml );
	
	_view->ShowPosition( _view->GetLastPosition() );
}

//////////////////////////////////////////////////////////////////////////////
// BMLFrame add some XML (impl)

void BMLFrame::AddXML( XMLElement* element, int depth )
{
	_view->BeginLeftIndent( depth * 40 );
	
	_view->BeginTextColour( wxColour( 128, 64, 64 ) );
	_view->WriteText( "<" );
	
	if ( ! element->GetNamespace().empty() )
	{
		_view->WriteText( element->GetNamespace() );
		_view->WriteText( ":" );
	}
	
	_view->WriteText( element->GetName() );
	_view->EndTextColour();
	
	XMLIterator it = element->GetChildren();
	
	for ( ; it ; ++ it )
	{
		XMLAttribute* att = it->IsAttribute();
		if ( NULL == att ) break;
		
		_view->WriteText( " " );
		_view->BeginTextColour( wxColour( 0, 128, 0 ) );
		
		if ( ! att->GetNamespace().empty() )
		{
			_view->WriteText( att->GetNamespace() );
			_view->WriteText( ":" );
		}
		
		_view->WriteText( att->GetName() );
		_view->WriteText( "=" );
		_view->EndTextColour();
		_view->BeginTextColour( wxColour( 0, 0, 255 ) );
		
		wxString value( att->GetValue() );
		value.Replace( "&", "&amp;" );
		value.Replace( "\"", "&quot;" );
		
		_view->WriteText( "\"" + value + "\"" );
		_view->EndTextColour();
	}
	
	if ( ! it )
	{
		_view->BeginTextColour( wxColour( 128, 64, 64 ) );
		_view->WriteText( "/>" );
		_view->EndTextColour();
		_view->EndLeftIndent();
		_view->Newline();
		return;
	}
	
	_view->BeginTextColour( wxColour( 128, 64, 64 ) );
	_view->WriteText( ">" );
	_view->EndTextColour();
	
	for ( bool nl = false ; it ; ++ it )
	{
		if ( XMLBody* body = it->IsBody() )
		{
			wxString value( body->GetValue() );
			value.Replace( "&", "&amp;" );
			value.Replace( "<", "&lt;" );
			value.Replace( ">", "&gt;" );
			
			_view->BeginTextColour( wxColour( 128, 128, 128 ) );
			_view->WriteText( value );
			_view->EndTextColour();
		}
		else if ( XMLElement* child = it->IsElement() )
		{
			if ( ! nl )
			{
				_view->EndLeftIndent();
				_view->Newline();
				nl = true;
			}
			
			AddXML( child, depth + 1 );
		}
	}
	
	if ( nl )
	{
		_view->BeginLeftIndent( depth * 40 );
	}
	
	_view->BeginTextColour( wxColour( 128, 64, 64 ) );
	_view->WriteText( "</" );
	
	if ( ! element->GetNamespace().empty() )
	{
		_view->WriteText( element->GetNamespace() );
		_view->WriteText( ":" );
	}
	
	_view->WriteText( element->GetName() );
	_view->WriteText( ">" );
	_view->EndTextColour();
	
	_view->EndLeftIndent();
	_view->Newline();
}

//////////////////////////////////////////////////////////////////////////////
// BMLFrame build the actor list

void BMLFrame::BuildActorList()
{
	while ( _actor->GetCount() > 0 ) _actor->Delete( 0 );
	
	if ( Application* app = Application::GetInstance() )
	{
		const list<Actor*>& actors = app->GetActors()->GetActors();
		
		for ( list<Actor*>::const_iterator it = actors.begin() ; it != actors.end() ; ++ it )
		{
			_actor->Append( ( *it )->GetName() );
		}
	}
}

//////////////////////////////////////////////////////////////////////////////
// BMLFrame command handlers

void BMLFrame::OnClose( wxCloseEvent& event )
{
	Destroy();
}

void BMLFrame::OnActivate( wxActivateEvent& event )
{
	event.Skip();
	BuildActorList();
}

void BMLFrame::OnCharHook( wxKeyEvent& event )
{
	if ( event.GetKeyCode() == WXK_RETURN )
	{
		wxCommandEvent cmd( wxEVT_COMMAND_BUTTON_CLICKED, ID_BML_CONSOLE );
		OnExecute( cmd );
		return;
	}
	
	event.Skip();
}

void BMLFrame::OnExecute( wxCommandEvent& event )
{
	string actor( _actor->GetValue() );
	
	if ( actor.empty() )
	{
		wxMessageBox( "You must select an actor name from the list." );
		_actor->SetFocus();
		return;
	}
	
	_last = _edit->GetValue();
	XMLElement* xml = XMLElement::FromString( _last.c_str() );
	XMLAutoDelete ad( xml );
	
	if ( NULL == xml )
	{
		wxMessageBox( "Your XML could not be parsed.  It may have syntax errors." );
		_edit->SetFocus();
		return;
	}
	
	Application* app = Application::GetInstance();
	
	if ( NULL == app )
	{
		wxMessageBox( "There is no project loaded.  Load a project first." );
		_edit->SetFocus();
		return;
	}
	
	if ( ! app->GetActors()->IsConnected() )
	{
		wxMessageBox( "SmartBody is not connected." );
		_edit->SetFocus();
		return;
	}
	
	if ( NULL == app->GetActors()->FindActor( actor.c_str() ) )
	{
		if ( ! xml->IsNamed( "vgs", "create" ) )
		{
			wxMessageBox( "Actor " + actor + " does not exist, and this is not a <vgs:create> request.\n\nYou must send requests to existing actors." );
			_actor->SetFocus();
			return;
		}
	}
	
	app->GetActors()->Send( actor.c_str(), xml );
	
	BuildActorList();
	
	_edit->SetValue( "" );
	_edit->SetFocus();
}

void BMLFrame::OnLast( wxCommandEvent& event )
{
	_edit->SetValue( _last );
	_edit->SetFocus();
}

void BMLFrame::OnTemplate( wxCommandEvent& event )
{
	int sel = _template->GetSelection();
	if ( sel == wxNOT_FOUND ) return;
	_template->SetSelection( wxNOT_FOUND );
	
	_edit->SetValue( _templates[ sel ] );
	_edit->SetFocus();
}
