//
// ScriptDialog.cpp
// Copyright Michael J Stokes, 2008.
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "Common.h"
#include "Editor.h"
#include "ScriptDialog.h"


IMPLEMENT_ABSTRACT_CLASS( ScriptDialog, wxDialog )

BEGIN_EVENT_TABLE( ScriptDialog, wxDialog )
	EVT_BUTTON( wxID_OK, ScriptDialog::OnOK )
END_EVENT_TABLE()


//////////////////////////////////////////////////////////////////////////////
// ScriptDialog construction

ScriptDialog::ScriptDialog( wxWindow* parent, XMLElement* block ) : wxDialog( parent, wxID_ANY, T("Script Block"), wxDefaultPosition )
{
	_block = block;
	
	if ( NULL == _block )
	{
		_block = new XMLElement( "vgs", "block" );
		_block->AddAttribute( "time", "0.0" );
		_block->AddAttribute( "actor", "guide" );
		_block->AddElement( "gaze" )->AddAttribute( "target", "ego" );
	}
	
	_edit = new wxRichTextCtrl( this, 1, wxEmptyString, wxDefaultPosition, wxSize( 0, 0 ), wxVSCROLL | wxWANTS_CHARS );
	_edit->SetMinSize( wxSize( 400, 200 ) );
	_edit->SetFont( wxFont( wxSystemSettings::GetFont( wxSYS_ANSI_FIXED_FONT ) ) );
	
	_edit->SetValue( _block->ToString( false, true ) );
	
	wxBoxSizer* sizer = new wxBoxSizer( wxVERTICAL );
	sizer->Add( _edit, 1, wxEXPAND | wxLEFT | wxTOP | wxRIGHT, 10 );
	sizer->AddSpacer( 16 );
	sizer->Add( CreateButtonSizer( wxOK | wxCANCEL ), 0, wxALIGN_CENTER | wxLEFT | wxBOTTOM | wxRIGHT, 10 );
	SetSizerAndFit( sizer );
}

//////////////////////////////////////////////////////////////////////////////
// ScriptDialog destruction

ScriptDialog::~ScriptDialog()
{
	_block->Delete();
}

//////////////////////////////////////////////////////////////////////////////
// ScriptDialog OK button

void ScriptDialog::OnOK( wxCommandEvent& event )
{
	XMLElement* block = XMLElement::FromString( _edit->GetValue() );
	XMLAutoDelete ad( block );
	
	if ( NULL == block )
	{
		wxMessageBox( "Your XML could not be parsed.  It may have syntax errors." );
		_edit->SetFocus();
		return;
	}
	else if ( ! block->IsNamed( "vgs", "block" ) )
	{
		wxMessageBox( "You must specify a <vgs:block> element." );
		_edit->SetFocus();
		return;
	}
	else if ( NULL == block->GetAttribute( "actor" ) )
	{
		wxMessageBox( "You must specify the \"actor\" attribute, for example <vgs:block actor=\"actor name\">." );
		_edit->SetFocus();
		return;
	}
	else if ( NULL == block->GetAttribute( "time" ) )
	{
		wxMessageBox( "You must specify the \"time\" attribute, for example <vgs:block time=\"1.0\">." );
		_edit->SetFocus();
		return;
	}
	
	_block->Delete();
	_block = ad.Detach();
	
	EndDialog( wxID_OK );
}
