//
// XPToolBar.h
// Copyright Michael J Stokes, 2009.
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#pragma once


class XPToolBar : public wxToolBarBase
{
// Selection
public:
	static wxToolBar*			Create( wxWindow* parent );
	
// Construction
public:
	XPToolBar();
	XPToolBar( wxWindow *parent, wxWindowID id = wxID_ANY, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = wxNO_BORDER | wxTB_HORIZONTAL, const wxString& name = wxToolBarNameStr );
	virtual ~XPToolBar();
	
// Operations
public:
	bool						Create( wxWindow *parent, wxWindowID id, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = wxNO_BORDER | wxTB_HORIZONTAL, const wxString& name = wxToolBarNameStr );
	bool						Realize();
	virtual wxToolBarToolBase*	FindToolForPosition( wxCoord x, wxCoord y ) const;

// Implementation
protected:
	void						Init();
	virtual wxToolBarToolBase*	CreateTool( int id, const wxString& label, const wxBitmap& bmpNormal, const wxBitmap& bmpDisabled, wxItemKind kind, wxObject *clientData, const wxString& shortHelp, const wxString& longHelp );
	virtual wxToolBarToolBase*	CreateTool( wxControl *control );
	virtual bool				DoInsertTool( size_t pos, wxToolBarToolBase *tool );
	virtual bool				DoDeleteTool( size_t pos, wxToolBarToolBase *tool );
	virtual void				DoEnableTool( wxToolBarToolBase *tool, bool enable );
	virtual void				DoToggleTool( wxToolBarToolBase *tool, bool toggle );
	virtual void				DoSetToggle( wxToolBarToolBase *tool, bool toggle );
	virtual wxSize				DoGetBestSize() const;
	
// Events
protected:
	void OnEraseBackground( wxEraseEvent& event );
	void OnPaint( wxPaintEvent& event );
	void OnMouseMove( wxMouseEvent& event );
	void OnMouseLeave( wxMouseEvent& event );
	void OnMouseDown( wxMouseEvent& event );
	void OnMouseUp( wxMouseEvent& event );
	void OnLostCapture( wxMouseCaptureLostEvent& event );
	
// Class
private:
	DECLARE_EVENT_TABLE()
	DECLARE_DYNAMIC_CLASS( XPToolBar )
	DECLARE_NO_COPY_CLASS( XPToolBar )
	
// Data
protected:
	wxColor				_backColour;
	wxPen*				_backPen;
	wxBrush*			_backBrush;
	wxPen*				_sepPen;
	wxColour			_selColour;
	wxPen*				_selPen;
	wxBrush*			_selBrush;
	wxColour			_shadowColour;
	wxBrush*			_shadowBrush;
	wxColour			_disabledColour;
protected:
	wxToolBarToolBase*	_hover;
	wxToolBarToolBase*	_down;
};
